import asyncio
from agentmake import agentmake, DEFAULT_AI_BACKEND, unpack_instruction_content, unpack_system_content
from agentmake.utils.text_wrapper import get_stream_event_text
from agentmake.utils.read_assistant_response import is_openai_style
from biblemateweb import DEFAULT_MESSAGES

async def stream_output(messages, user_request, cancel_event=None, system=None, **kwargs):
    def get_next_chunk(iterator):
        """
        Runs in a separate thread. 
        Returns the next item, or None if the iterator is exhausted.
        """
        try:
            return next(iterator)
        except StopIteration:
            return None
        except Exception as e:
            return e  # Return the error to be handled in the main loop

    if "instruction" in kwargs:
        instruction_content = kwargs.pop("instruction")
        instruction_content = unpack_instruction_content(instruction_content)
        # refine user request
        user_request = instruction_content + "\n" + user_request

    if system == "auto":
        system = await stream_output(DEFAULT_MESSAGES, user_request, cancel_event, system="bible/create_agent", **kwargs)
        if not system or system.strip() == "[NO_CONTENT]":
            return None
        else:
            # refine response
            system = system.replace("should:", "should:\n")
            system = system.replace("examples:", "examples:\n")
            if system.startswith("```agent\n"):
                system = system[9:]
            if system.endswith("```"):
                system = system[:-3].strip()
    elif system is not None:
        system_content = unpack_system_content(system)
        system = None
        # refine user request
        user_request = f"""---

START OF YOUR NEW ROLE

---

{system_content}

---

START OF MY REQUEST

---

{user_request}"""

    # Print loading text
    print("Loading...", end="\r", flush=True)
    await asyncio.sleep(0)

    backend=kwargs.pop("backend", DEFAULT_AI_BACKEND)
    model=kwargs.pop("model", None)
    api_key=kwargs.pop("api_key", None)
    api_endpoint=kwargs.pop("api_endpoint", None)
    max_tokens=kwargs.pop("max_tokens", None)
    context_window=kwargs.pop("context_window", None)
    temperature=kwargs.pop("temperature", None)

    # run completion
    text_chunks = ""
    completion = await asyncio.to_thread(
        agentmake, 
        messages, 
        system=system,
        backend=backend, 
        model=model, 
        api_key=api_key,
        api_endpoint=api_endpoint,
        max_tokens=max_tokens,
        context_window=context_window,
        temperature=temperature,
        follow_up_prompt=user_request, 
        stream=True, 
        print_on_terminal=False, 
        stream_events_only=True, 
        **kwargs,
    )

    print("Running...  ", end="\r", flush=True)

    try:
        while cancel_event is None or not cancel_event.is_set():
            event = await asyncio.to_thread(get_next_chunk, completion)

            if event is None:
                break
            elif isinstance(event, Exception):
                print(f"\nStream interrupted: {str(event)}")
                break

            actual_backend = backend if backend else DEFAULT_AI_BACKEND
            if text_chunk := get_stream_event_text(event, openai_style=is_openai_style(actual_backend)):
                text_chunks += text_chunk
                print(text_chunk, end="", flush=True)
                await asyncio.sleep(0)

        if cancel_event is not None and cancel_event.is_set():
            print("\n[Cancelled!]")
        else:
            print() # Print newline when done

    except Exception as e:
        print(f"\n[Error: {str(e)}]")
        if cancel_event is not None:
            cancel_event.set()

    if cancel_event is not None and cancel_event.is_set():
        cancel_event = None
        return None
    
    return text_chunks.replace(" ", " ").replace("‑", "-")



import asyncio, argparse, sys
from agentmake import SUPPORTED_AI_BACKENDS, DEFAULT_AI_BACKEND
from bibleagent.agent import bible_agent

parser = argparse.ArgumentParser(description = f"""BibleMate AI Agent""")
parser.add_argument("default", nargs="*", default=None, help="user request")
parser.add_argument("-l", "--language", action="store", dest="language", choices=["eng", "tc", "sc"], help="language option")
parser.add_argument("-b", "--backend", action="store", dest="backend", choices=SUPPORTED_AI_BACKENDS, help="AI backend option")
parser.add_argument("-m", "--model", action="store", dest="model", help="AI model option")
parser.add_argument("-k", "--api_key", action="store", dest="api_key", help="API key option")
parser.add_argument("-e", "--api_endpoint", action="store", dest="api_endpoint", help="API endpoint option")
parser.add_argument("-mt", "--max_tokens", action="store", dest="max_tokens", type=int, help="max tokens option")
parser.add_argument("-cw", "--context_window", action="store", dest="context_window", type=int, help="context window option")
parser.add_argument("-t", "--temperature", action="store", dest="temperature", type=float, help="temperature option")
parser.add_argument("-p", "--improve_prompt", action="store_true", dest="improve_prompt", help="improve user prompt")
parser.add_argument("-d", "--developer", action="store_true", dest="developer", help="developer mode")
parser.add_argument("-md", "--md_export", action="store_true", dest="md_export", help="export outputs in markdown format")
parser.add_argument("-docx", "--docx_export", action="store_true", dest="docx_export", help="export outputs in docx format")
parser.add_argument("-o", "--output_directory", action="store", dest="output_directory", help="output directory")
args = parser.parse_args()

async def main_async():
    print("Starting BibleMate AI Agent...")
    kwargs = {}
    kwargs["backend"] = args.backend if args.backend else DEFAULT_AI_BACKEND
    kwargs["model"] = args.model if args.model else None
    kwargs["api_key"] = args.api_key if args.api_key else None
    kwargs["api_endpoint"] = args.api_endpoint if args.api_endpoint else None
    kwargs["max_tokens"] = args.max_tokens if args.max_tokens else None
    kwargs["context_window"] = args.context_window if args.context_window else None
    kwargs["temperature"] = args.temperature if args.temperature else None
    
    user_requests = []
    if args.default is not None:
        user_requests.append(" ".join(args.default))
    stdin_text = sys.stdin.read() if not sys.stdin.isatty() else ""
    if stdin_text:
        user_requests.append(stdin_text.strip())
    
    await bible_agent(
        request="\n\n".join(user_requests) if user_requests else "",
        language=args.language if args.language else "eng",
        improve_prompt=args.improve_prompt if args.improve_prompt else False,
        md_export=args.md_export if args.md_export else False,
        docx_export=args.docx_export if args.docx_export else False,
        output_directory=args.output_directory if args.output_directory else "",
        developer=args.developer if args.developer else False,
        **kwargs,
    )
    print("Finished")

def main():
    asyncio.run(main_async())

if __name__ == "__main__":
    main()
